@@ -1,4 +1,5 @@
-{-# LANGUAGE FlexibleContexts, UndecidableInstances, CPP #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE FlexibleContexts, UndecidableInstances #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Data.SafeCopy.Instances where
 
@@ -31,7 +32,7 @@ import           Data.Time.Clock (DiffTime, NominalDiffTime, UniversalTime(..),
 import           Data.Time.Clock.TAI (AbsoluteTime, taiEpoch, addAbsoluteTime, diffAbsoluteTime)
 import           Data.Time.LocalTime (LocalTime(..), TimeOfDay(..), TimeZone(..), ZonedTime(..))
 import qualified Data.Tree as Tree
-#if __GLASGOW_HASKELL__ >= 707
+#if MIN_VERSION_base(4,7,0)
 import           Data.Typeable hiding (Proxy)
 #else
 import           Data.Typeable
@@ -39,6 +40,11 @@ import           Data.Typeable
 import           Data.Word
 import           System.Time (ClockTime(..), TimeDiff(..), CalendarTime(..), Month(..))
 import qualified System.Time as OT
+import qualified Data.Vector as V
+import qualified Data.Vector.Generic as VG
+import qualified Data.Vector.Primitive as VP
+import qualified Data.Vector.Storable as VS
+import qualified Data.Vector.Unboxed as VU
 
 instance SafeCopy a => SafeCopy (Prim a) where
   kind = primitive
@@ -352,8 +358,8 @@ instance SafeCopy CalendarTime where
                              pico   <- get
                              wday   <- safeGet
                              yday   <- get
-                             tzname <- get
-                             tz     <- safeGet
+                             tzname <- safeGet
+                             tz     <- get
                              dst    <- get
                              return (CalendarTime year month day hour mins sec pico wday yday tzname tz dst)
     putCopy t = contain $ do put     (ctYear t)
@@ -372,18 +378,32 @@ instance SafeCopy CalendarTime where
 typeName :: Typeable a => Proxy a -> String
 typeName proxy = show (typeOf (undefined `asProxyType` proxy))
 
-#if __GLASGOW_HASKELL__ < 707
 typeName1 :: (Typeable1 c) => Proxy (c a) -> String
 typeName1 proxy = show (typeOf1 (undefined `asProxyType` proxy))
 
 typeName2 :: (Typeable2 c) => Proxy (c a b) -> String
 typeName2 proxy = show (typeOf2 (undefined `asProxyType` proxy))
 
-#else
-typeName1 :: (Typeable c) => Proxy (c a) -> String
-typeName1 proxy = show (typeOf1 (undefined `asProxyType` proxy))
+getGenericVector :: (SafeCopy a, VG.Vector v a) => Contained (Get (v a))
+getGenericVector = contain $ do n <- get
+                                getSafeGet >>= VG.replicateM n
 
-typeName2 :: (Typeable c) => Proxy (c a b) -> String
-typeName2 proxy = show (typeOf2 (undefined `asProxyType` proxy))
-#endif
+putGenericVector :: (SafeCopy a, VG.Vector v a) => v a -> Contained Put
+putGenericVector v = contain $ do put (VG.length v)
+                                  getSafePut >>= VG.forM_ v
 
+instance SafeCopy a => SafeCopy (V.Vector a) where
+    getCopy = getGenericVector
+    putCopy = putGenericVector
+
+instance (SafeCopy a, VP.Prim a) => SafeCopy (VP.Vector a) where
+    getCopy = getGenericVector
+    putCopy = putGenericVector
+
+instance (SafeCopy a, VS.Storable a) => SafeCopy (VS.Vector a) where
+    getCopy = getGenericVector
+    putCopy = putGenericVector
+
+instance (SafeCopy a, VU.Unbox a) => SafeCopy (VU.Vector a) where
+    getCopy = getGenericVector
+    putCopy = putGenericVector
